#!/usr/bin/env python

"""
Handlers for a resource.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.content import Handler
from imiptools.data import get_address, get_uri, to_part
from imiptools.handlers.common import CommonFreebusy

class ResourceHandler(Handler):

    "Handling mechanisms specific to resources."

    def _record_and_respond(self, handle_for_attendee):

        oa = self.require_organiser_and_attendees()
        if not oa:
            return None

        organiser_item, attendees = oa

        # Process each attendee separately.

        calendar = []

        for attendee, attendee_attr in attendees.items():

            # Check for event using UID.

            if not self.have_new_object(attendee):
                continue

            # Collect response objects produced when handling the request.

            response = handle_for_attendee(attendee, attendee_attr)
            if response:
                calendar.append(response)

        return calendar

    def _schedule_for_attendee(self, attendee, attendee_attr):

        # If newer than any old version, discard old details from the
        # free/busy record and check for suitability.

        periods = self.get_periods()
        freebusy = self.store.get_freebusy(attendee)
        scheduled = self.can_schedule(freebusy, periods)

        attendee_attr["PARTSTAT"] = scheduled and "ACCEPTED" or "DECLINED"
        if self.messenger and self.messenger.sender != get_address(attendee):
            attendee_attr["SENT-BY"] = get_uri(self.messenger.sender)

        # Make a version of the request with just this attendee.

        self.obj["ATTENDEE"] = [(attendee, attendee_attr)]

        # Update the DTSTAMP.

        self.update_dtstamp()

        event = self.obj.to_node()
        self.store.set_event(attendee, self.uid, event)

        # Only update free/busy details if the event is scheduled.

        if scheduled:
            self.update_freebusy(freebusy, attendee, periods)
        else:
            self.remove_from_freebusy(freebusy, attendee)

        if self.publisher:
            self.publisher.set_freebusy(attendee, freebusy)

        return event

    def _cancel_for_attendee(self, attendee, attendee_attr):

        freebusy = self.store.get_freebusy(attendee)
        self.remove_from_freebusy(freebusy, attendee)

        if self.publisher:
            self.publisher.set_freebusy(attendee, freebusy)

        return None

class Event(ResourceHandler):

    "An event handler."

    def add(self):
        pass

    def cancel(self):

        "Cancel attendance for attendees."

        self._record_and_respond(self._cancel_for_attendee)
        return None

    def counter(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def declinecounter(self):

        """
        Since this handler does not send counter proposals, it will not handle
        replies to such proposals.
        """

        pass

    def publish(self):
        pass

    def refresh(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def request(self):

        """
        Respond to a request by preparing a reply containing accept/decline
        information for each indicated attendee.

        No support for countering requests is implemented.
        """

        response = self._record_and_respond(self._schedule_for_attendee)
        if response:
            return [(map(get_address, self.obj.get_values("ORGANIZER")), to_part("REPLY", response))]
        else:
            return None

class Freebusy(ResourceHandler, CommonFreebusy):

    "A free/busy handler."

    def publish(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    # request provided by CommonFreeBusy.request

class Journal(ResourceHandler):

    "A journal entry handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def publish(self):
        pass

class Todo(ResourceHandler):

    "A to-do item handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def declinecounter(self):

        """
        Since this handler does not send counter proposals, it will not handle
        replies to such proposals.
        """

        pass

    def publish(self):
        pass

    def refresh(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def request(self):
        pass

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
